--********************************************************
--*                                                      *
--* Information_Collector                                * BODY
--*                                                      *
--********************************************************
with Ada.Unchecked_Deallocation; -- from standard Ada environment
package body Information_Collector is

  -- Internal data structures
    type Information_Packet_Capsule;
    type Information_Packet_Iterator is access Information_Packet_Capsule;

    type Information_Packet_Capsule is record
      Data : Information_Packet;
      Next : Information_Packet_Iterator := null;
    end record;

    Internal_Iterator : Information_Packet_Iterator := null;

    type Information_Packet_Pointer_Vector is array (INTEGER range <>)
      of Information_Packet_Iterator;

    type Information_Packet_Pointer_Vector_Pointer is access
      Information_Packet_Pointer_Vector;

    First_Packet : Information_Packet_Iterator := null;
    Last_Packet  : Information_Packet_Iterator := null;

  --********************************************************
  --*                                                      *
  --* Information_Collector.Quick_Sort                     *
  --*                                                      *
  --********************************************************
  generic
    type ITEM is private;
    type INDEX is (<>);
    type ROW is array (INDEX range <>) of ITEM;
    with function "<" (X, Y : ITEM) return BOOLEAN is <>;
  procedure Quick_Sort (A : in out ROW);
  --| Purpose
  --| Sort the one-dimensional array A using the Quick Sort
  --| algorithm.
  --|
  --| Exceptions (none)
  --| Notes (none)
  --|
  --| Modifications
  --| Author: John Anderson, TI
  --| Modified: Richard Conn, Monmouth University - extracted
  --|   from CS Parts and modified for Information_Collector

  --********************************************************
  --*                                                      *
  --* Information_Collector.Quick_Sort (body)              *
  --*                                                      *
  --********************************************************
  procedure Quick_Sort (A : in out ROW) is

    procedure Quick_Sort_INTERNAL (L, R : INDEX) is

        I, J : INDEX;
        X    : ITEM;
        TEMP : ITEM;

    begin

        I := L;
        J := R;

        X := A (INDEX'VAL ((INDEX'POS (L) + INDEX'POS (R)) / 2));

        MAIN:
        loop

            while A (I) < X loop
                I := INDEX'SUCC (I);
            end loop;

            while X < A (J) loop
                J := INDEX'PRED (J);
            end loop;

            if I <= J then
                TEMP := A(I);
                A(I) := A(J);
                A(J) := TEMP;

                begin
                    I := INDEX'SUCC (I);
                    J := INDEX'PRED (J);
                exception
                    when CONSTRAINT_ERROR =>
                        null; -- necessary to avoid exception raising
                end;

            end if;

            exit when I > J;

        end loop MAIN;

        if L < J then
            Quick_Sort_INTERNAL (L, J);
        end if;

        if I < R then
            Quick_Sort_INTERNAL (I, R);
        end if;

    end Quick_Sort_INTERNAL;

  begin

    Quick_Sort_INTERNAL (A'FIRST, A'LAST);

  exception
    when others =>
      raise INTERNAL_QUICK_SORT_ERROR;
  end Quick_Sort;

  --********************************************************
  --*                                                      *
  --* Information_Collector.Add_Information_Packet         *
  --*                                                      *
  --********************************************************
  procedure Add_Information_Packet (New_Packet : in Information_Packet) is
  --| PURPOSE
  --| Add an information packet to the internal data structure.
  --|
  --| RAISES
  --|   DYNAMIC_MEMORY_EXHAUSTED
  begin -- Add_Information_Packet
    if First_Packet = null then
      First_Packet     := new Information_Packet_Capsule;
      Last_Packet      := First_Packet;
    else
      Last_Packet.Next := new Information_Packet_Capsule;
      Last_Packet      := Last_Packet.Next;
    end if;
    Last_Packet.Data := New_Packet;
  exception
    when others =>
      raise DYNAMIC_MEMORY_EXHAUSTED;
  end Add_Information_Packet;

  --********************************************************
  --*                                                      *
  --* Information_Collector.Get_Last_Information_Packet    *
  --*                                                      *
  --********************************************************
  function Get_Last_Information_Packet return Information_Packet is
  --| PURPOSE
  --| Return the last information packet in the data structure.
  --| This packet is either the information packet that was last
  --| added to the internal data structure via Add_Information_Packet
  --| or the information packet that is the last one in the list
  --| as a result of a call to Sort_Information_Packets.
  --|
  --| RAISES
  --|   INTERNAL_DATA_STRUCTURE_IS_EMPTY
  begin -- Get_Last_Information_Packet
    if First_Packet = null then
      raise INTERNAL_DATA_STRUCTURE_IS_EMPTY;
    end if;
    return Last_Packet.Data;
  end Get_Last_Information_Packet;

  --********************************************************
  --*                                                      *
  --* Information_Collector.Update_Last_Information_Packet *
  --*                                                      *
  --********************************************************
  procedure Update_Last_Information_Packet
    (Revised_Packet : in Information_Packet) is
  --| PURPOSE
  --| Replace the last information packet in the data structure.
  --| This packet is either the information packet that was last
  --| added to the internal data structure via Add_Information_Packet
  --| or the information packet that is the last one in the list
  --| as a result of a call to Sort_Information_Packets.
  --|
  --| RAISES
  --|   INTERNAL_DATA_STRUCTURE_IS_EMPTY
  begin -- Update_Last_Information_Packet
    if First_Packet = null then
      raise INTERNAL_DATA_STRUCTURE_IS_EMPTY;
    end if;
    Last_Packet.Data := Revised_Packet;
  end Update_Last_Information_Packet;

  --********************************************************
  --*                                                      *
  --* Information_Collector.Sort_Information_Packets       *
  --*                                                      *
  --********************************************************
  procedure Sort_Information_Packets is
  --| PURPOSE
  --| Sort the information packets in the internal data structure.
  --|
  --| RAISES
  --|   DYNAMIC_MEMORY_EXHAUSTED

    Rover : Information_Packet_Iterator := First_Packet;
    Count : INTEGER := 0;
    IP_VP : Information_Packet_Pointer_Vector_Pointer := null;

    --************************************************************
    --*                                                          *
    --* Information_Collector.Sort_Information_Packets.Free      *
    --*                                                          *
    --************************************************************
    procedure Free is new Ada.Unchecked_Deallocation
      (Object => Information_Packet_Pointer_Vector,
       Name   => Information_Packet_Pointer_Vector_Pointer);

    --************************************************************
    --*                                                          *
    --* Information_Collector.Sort_Information_Packets.Less_Than *
    --*                                                          *
    --************************************************************
    function Less_Than (L, R : in Information_Packet_Iterator)
      return BOOLEAN is
    begin -- Less_Than
      return Information_Collector."<" (L.Data, R.Data);
        -- this is the "<" function provided as a generic formal
        -- parameter
    end Less_Than;

    --************************************************************
    --*                                                          *
    --* Information_Collector.Sort_Information_Packets.SIP_Sort  *
    --*                                                          *
    --************************************************************
    procedure SIP_Sort is new Quick_Sort
      (Item  => Information_Packet_Iterator,
       Index => INTEGER,
       Row   => Information_Packet_Pointer_Vector,
       "<"   => Less_Than);

    --************************************************************
    --*                                                          *
    --* Information_Collector.Sort_Information_Packets (body)    *
    --*                                                          *
    --************************************************************
  begin -- Sort_Information_Packets
    if First_Packet /= null and then
       First_Packet.Next /= null then
           -- do this only if 2 or more packets are in the list

      -- Set Count to number of packets in linked list
      while Rover /= null loop
        Count := Count + 1;
        Rover := Rover.Next;
      end loop;

      -- Create an array of packet pointers
      IP_VP := new Information_Packet_Pointer_Vector (1..Count);

      -- Populate the array with pointers to the linked list
      Rover := First_Packet;
      Count := 0;
      while Rover /= null loop
        Count            := Count + 1;
        IP_VP.all(Count) := Rover;
        Rover            := Rover.Next;
      end loop;

      -- Perform the sort
      SIP_Sort (IP_VP.all);

      -- Reset the linked list
      First_Packet := IP_VP.all(1);
      Rover        := First_Packet;
      for I in 2 .. Count loop
        Rover.Next := IP_VP.all(I);
        Rover      := Rover.Next;
      end loop;
      Rover.Next  := null;
      Last_Packet := Rover;

      -- Free array of pointers
      Free (IP_VP);

    end if;

  exception
    when INTERNAL_QUICK_SORT_ERROR => raise;
    when others                    => raise DYNAMIC_MEMORY_EXHAUSTED;
  end Sort_Information_Packets;

  --********************************************************
  --*                                                      *
  --* Information_Collector.Reset_Iterator                 *
  --*                                                      *
  --********************************************************
  procedure Reset_Iterator is
  --| PURPOSE
  --| Reset the internal Information_Packet_Iterator to
  --| the first of the information packets in the internal
  --| data structure.
  --|
  --| RAISES
  --|   Nothing
  begin -- Reset_Iterator
    Internal_Iterator := First_Packet;
  end Reset_Iterator;

  --********************************************************
  --*                                                      *
  --* Information_Collector.Is_End_of_Packets              *
  --*                                                      *
  --********************************************************
  function Is_End_of_Packets return BOOLEAN is
  --| PURPOSE
  --| Return TRUE iff there are no more packets in the internal
  --| data structure.
  --|
  --| RAISES
  --|   Nothing
  begin -- Is_End_of_Packets
    return Internal_Iterator = null;
  end Is_End_of_Packets;

  --********************************************************
  --*                                                      *
  --* Information_Collector.Fetch_Next_Information_Packet  *
  --*                                                      *
  --********************************************************
  procedure Fetch_Next_Information_Packet
    (Next_Packet :    out Information_Packet) is
  --| PURPOSE
  --| Return the next packet from the internal data structure based
  --| on the Iterator.  Advance the Iterator.
  --|
  --| RAISES
  --|   NO_MORE_PACKETS
  begin -- Fetch_Next_Information_Packet
    if Internal_Iterator = null then
      raise NO_MORE_PACKETS;
    else
      Next_Packet := Internal_Iterator.Data;
      Internal_Iterator := Internal_Iterator.Next;
    end if;
  end Fetch_Next_Information_Packet;

end Information_Collector;
