--********************************************************
--*                                                      *
--* Information_Collector                                * SPEC
--*                                                      *
--********************************************************
generic
  type Information_Packet is private;
  with function "<" (L, R : in Information_Packet) return BOOLEAN;
package Information_Collector is
--| PURPOSE
--| Collect information packets into an internal data structure
--| that can be sorted and traversed.
--|
--| AUTHOR
--| Richard Conn  27 Feb 96  Initial version based on previous work

  --********************************************************
  --*                                                      *
  --* Information_Collector.Add_Information_Packet         *
  --*                                                      *
  --********************************************************
  procedure Add_Information_Packet (New_Packet : in Information_Packet);
  --| PURPOSE
  --| Add an information packet to the internal data structure.
  --|
  --| RAISES
  --|   DYNAMIC_MEMORY_EXHAUSTED

  --********************************************************
  --*                                                      *
  --* Information_Collector.Get_Last_Information_Packet    *
  --*                                                      *
  --********************************************************
  function Get_Last_Information_Packet return Information_Packet;
  --| PURPOSE
  --| Return the last information packet in the data structure.
  --| This packet is either the information packet that was last
  --| added to the internal data structure via Add_Information_Packet
  --| or the information packet that is the last one in the list
  --| as a result of a call to Sort_Information_Packets.
  --|
  --| RAISES
  --|   INTERNAL_DATA_STRUCTURE_IS_EMPTY

  --********************************************************
  --*                                                      *
  --* Information_Collector.Update_Last_Information_Packet *
  --*                                                      *
  --********************************************************
  procedure Update_Last_Information_Packet
    (Revised_Packet : in Information_Packet);
  --| PURPOSE
  --| Replace the last information packet in the data structure.
  --| This packet is either the information packet that was last
  --| added to the internal data structure via Add_Information_Packet
  --| or the information packet that is the last one in the list
  --| as a result of a call to Sort_Information_Packets.
  --|
  --| RAISES
  --|   INTERNAL_DATA_STRUCTURE_IS_EMPTY

  --********************************************************
  --*                                                      *
  --* Information_Collector.Sort_Information_Packets       *
  --*                                                      *
  --********************************************************
  procedure Sort_Information_Packets;
  --| PURPOSE
  --| Sort the information packets in the internal data structure.
  --|
  --| RAISES
  --|   DYNAMIC_MEMORY_EXHAUSTED
  --|   INTERNAL_QUICK_SORT_ERROR (should never be raised)

  --********************************************************
  --*                                                      *
  --* Information_Collector.Reset_Iterator                 *
  --*                                                      *
  --********************************************************
  procedure Reset_Iterator;
  --| PURPOSE
  --| Reset the internal Information_Packet_Iterator to
  --| the first of the information packets in the internal
  --| data structure.
  --|
  --| RAISES
  --|   Nothing

  --********************************************************
  --*                                                      *
  --* Information_Collector.Is_End_of_Packets              *
  --*                                                      *
  --********************************************************
  function Is_End_of_Packets return BOOLEAN;
  --| PURPOSE
  --| Return TRUE iff there are no more packets in the internal
  --| data structure.
  --|
  --| RAISES
  --|   Nothing

  --********************************************************
  --*                                                      *
  --* Information_Collector.Fetch_Next_Information_Packet  *
  --*                                                      *
  --********************************************************
  procedure Fetch_Next_Information_Packet
    (Next_Packet :    out Information_Packet);
  --| PURPOSE
  --| Return the next packet from the internal data structure based
  --| on the Iterator.  Advance the Iterator.
  --|
  --| RAISES
  --|   NO_MORE_PACKETS

  -- External exceptions (may be raised as indicated above)
    DYNAMIC_MEMORY_EXHAUSTED,
    INTERNAL_DATA_STRUCTURE_IS_EMPTY,
    NO_MORE_PACKETS
                                         : exception;

  -- Internal exceptions (should never be raised)
    INTERNAL_QUICK_SORT_ERROR : exception;

end Information_Collector;
