-- ****************************************************
-- *                                                  *
-- *  Time_Keeper                                     * BODY
-- *                                                  *
-- ****************************************************
with Ada.Calendar;
package body Time_Keeper is

  type Integer_Duration is range -86_400 .. 86_400;
  subtype Natural_Duration is Integer_Duration
    range 0 .. Integer_Duration'LAST;

  -- Month names
  Max_Month_Name_Length : constant POSITIVE  := 9;
  subtype Month_Name is STRING (1 .. Max_Month_Name_Length);
  type Month_Name_Vector is array (INTEGER range <>) of
    Month_Name;
  Month_Names : constant Month_Name_Vector (Ada.Calendar.MONTH_NUMBER) :=
    ("January  ", "February ", "March    ", "April    ", "May      ",
     "June     ", "July     ", "August   ", "September", "October  ",
     "November ", "December ");

  --*****************************************************
  --*                                                   *
  --* Time_Keeper.Compute_Day_of_Week                   *
  --*                                                   *
  --*****************************************************
  function Compute_Day_of_Week (Tod_Value : in Ada.Calendar.TIME)
    return STRING is
    --| Purpose
    --| This function was designed by A. Spencer Peterson, SEI.
    --|
    --| The following constant hardcodes the algorithm to work at the
    --| reference point of 1984.  Other hardcoded constants in the
    --| code nail the exact day, 1/1/84, to Sunday.  If the reference
    --| point is changed, then so must the day names returned.

    Ref_Year : constant Ada.Calendar.YEAR_NUMBER := 1984;

    Number_of_Days_in_a_Leap_Year     : constant Natural_Duration := 366;
    Number_of_Days_in_a_Normal_Year   : constant Natural_Duration := 365;
    Number_of_Days_in_Feb_Leap_Year   : constant Natural_Duration :=  29;
    Number_of_Days_in_Feb_Normal_Year : constant Natural_Duration :=  28;
    Number_of_Days_in_Long_Months     : constant Natural_Duration :=  31;
    Number_of_Days_in_Short_Months    : constant Natural_Duration :=  30;

    February  : constant POSITIVE :=   2;
    April     : constant POSITIVE :=   4;
    June      : constant POSITIVE :=   6;
    September : constant POSITIVE :=   9;
    November  : constant POSITIVE :=  11;

    Number_of_Days_in_a_Week : constant := 7;

    subtype Number_of_Days_Type is Integer_Duration
      range -Number_of_Days_in_a_Week + 1 .. Number_of_Days_in_a_Week - 1;

    First_Year,
    Last_Year,
    Input_Year           : Ada.Calendar.YEAR_NUMBER;
    Month_Count,
    Input_Month          : Ada.Calendar.MONTH_NUMBER;
    Input_Day            : Ada.Calendar.DAY_NUMBER;
    After_Ref_Year       : BOOLEAN;
    Constrained_Num_Days : Number_of_Days_Type;
    Number_of_Days       : Integer_Duration := 0;

    function Leap_Year (In_Year : in Ada.Calendar.YEAR_NUMBER) return BOOLEAN is
      Leap_Year_Century : constant POSITIVE := 400;
      Leap_Year_Offset  : constant POSITIVE :=   4;
      Century           : constant POSITIVE := 100;
    begin
      return (In_Year rem Leap_Year_Century = 0) or
             ((In_Year rem Leap_Year_Offset = 0) and
              (In_Year rem Century /= 0));
    end Leap_Year;

  begin  -- Compute_Day_of_Week
    -- Decode the Ada.Calendar.TIME into its subcomponents.

    Input_Year  := Ada.Calendar.YEAR (Tod_Value);
    Input_Month := Ada.Calendar.MONTH (Tod_Value);
    Input_Day   := Ada.Calendar.DAY (Tod_Value);

    -- Start of the algorithm follows below.

    if Input_Year < Ref_Year then
      After_Ref_Year := FALSE;
      First_Year     := Input_Year;
      Last_Year      := Ref_Year;
    else
      After_Ref_Year := TRUE;
      First_Year     := Ref_Year;
      Last_Year      := Input_Year;
    end if;

    while First_Year < Last_Year loop
      if Leap_Year (First_Year) then
        if After_Ref_Year then
          Number_of_Days := Number_of_Days + Number_of_Days_in_a_Leap_Year;
        else
          Number_of_Days := Number_of_Days - Number_of_Days_in_a_Leap_Year;
        end if;
      elsif After_Ref_Year then
        Number_of_Days := Number_of_Days + Number_of_Days_in_a_Normal_Year;
      else
        Number_of_Days := Number_of_Days - Number_of_Days_in_a_Normal_Year;
      end if;

      First_Year := First_Year + 1;
    end loop;

    Month_Count := 1;

    while Month_Count < Input_Month loop
      case Month_Count is
        when February =>
          if Leap_Year (Input_Year) then
            Number_of_Days := Number_of_Days + Number_of_Days_in_Feb_Leap_Year;
          else
            Number_of_Days := Number_of_Days +
              Number_of_Days_in_Feb_Normal_Year;
          end if;
        when April | June | September | November =>
          Number_of_Days := Number_of_Days + Number_of_Days_in_Short_Months;
        when others =>
          Number_of_Days := Number_of_Days + Number_of_Days_in_Long_Months;
      end case;

      Month_Count := Month_Count + 1;
    end loop;

    Constrained_Num_Days := (Number_of_Days +
      Integer_Duration (Input_Day)) rem Number_of_Days_in_a_Week;

    case Constrained_Num_Days is
      when -6 => return "Sunday";
      when -5 => return "Monday";
      when -4 => return "Tuesday";
      when -3 => return "Wednesday";
      when -2 => return "Thursday";
      when -1 => return "Friday";
      when  0 => return "Saturday";
      when  1 => return "Sunday";  -- The algorithm is hardcoded on this day.
      when  2 => return "Monday";
      when  3 => return "Tuesday";
      when  4 => return "Wednesday";
      when  5 => return "Thursday";
      when  6 => return "Friday";
    end case;
  end Compute_Day_of_Week;

  --*****************************************************
  --*                                                   *
  --* Time_Keeper.Trim                                  *
  --*                                                   *
  --*****************************************************
  function Trim (S : in STRING) return STRING is
  --| Purpose
  --| The function below trims trailing blanks from a string.
    Stop  : NATURAL := S'LAST;
  begin -- Trim
    for I in S'RANGE loop
      if S(I) > ' ' then
        Stop := I;
      end if;
    end loop;
    return S(S'FIRST .. Stop);
  end Trim;

  --*****************************************************
  --*                                                   *
  --* Time_Keeper.Convert                               *
  --*                                                   *
  --*****************************************************
  function Convert (Tod_Value : in Ada.Calendar.TIME) return STRING is
  --| Purpose
  --| The function below converts an internal Ada.Calendar.TIME value to
  --| an external STRING value.

    Year               : Ada.Calendar.YEAR_NUMBER;
    Month              : Ada.Calendar.MONTH_NUMBER;
    Day                : Ada.Calendar.DAY_NUMBER;
    Seconds            : Ada.Calendar.DAY_DURATION;

  begin  -- Convert
    -- Disect internal format into its components for our own use.
    Ada.Calendar.SPLIT (Tod_Value, Year, Month, Day, Seconds);

    -- Return concatenated string value
    return Compute_Day_of_Week(Tod_Value) & "," &
           NATURAL_DURATION'IMAGE(NATURAL_DURATION(Day)) & " " &
           Trim(Month_Names(Month)) &
           NATURAL_DURATION'IMAGE(NATURAL_DURATION(Year));
  end Convert;

  --*****************************************************
  --*                                                   *
  --* Time_Keeper.Current_Day_and_Date                  *
  --*                                                   *
  --*****************************************************
  function Current_Day_and_Date return STRING is
  begin -- Current_Day_and_Date
    return Convert(Ada.Calendar.Clock);
  end Current_Day_and_Date;

end Time_Keeper;
